#include <pic.h>
/*
//This is the software UART to read data at 9600 baud, all on T0CKI on a 16Fxxx part. For this to work as the UART input, we use the TMR0 interrupts as a edge detector for the start signal, and as a bit timer once start is detected. The RS232 data input (inverted) is on T0CKI. This system is based on a crystal clock running at 4MHz. The result is saved in UART_input_byte, and the outside world knows when data are available when flags.UART_data_valid (a global variable defined elsewhere) gets set. You should clear this flag when the input buffer is read in the outside world. And note: there is no check here that it won't be overwritten if the outside world has not read it!

The working principle is that TMR0 is either waiting for a start pulse on RA4, or is ticking away at the bit rate to sample the input on RA4 at the desired times. The mode is defined by the control bits for TMR0, as shown in these two OPTION register configuration words, below. start_OPTION_config is waiting for TMR0 to clock to zero from 0xFF, and fast_OPTION_config is using TMR0 to count off 1.5 or 1.0 bit times. The number of bits (8) is defined by the mask byte going to zero.

TMR0 interrupt must be enabled, and note that the prescaler is allocated to the watchdog timer so TMR0 runs at full speed. You could change that for slower baud rates. And don't forget to configure RA4 as an input!

*/
#define start_OPTION_config 0b10111001	//this is for waiting for start.
/*pull-ups disabled, falling edge of INT0 pin (ignored), T0CS = 1 to wait for start bit, high-to-low transition on T0CKI (START = 0, STOP = 1), prescaler allocated to WDT so TMR0 ticks up on first edge from -1.
*/

#define fast_OPTION_config 0b10011001	//this for inputting bits.
/*pull-ups disabled, falling edge of INT0 pin (ignored), T0CS = 0 to wait for TMR0 timeout, prescaler allocated to WDT so TMR0 ticks UP from pre-loaded bit time at full speed.
*/


#define bit_time_and_a_half 154	//(156-2 for TMR0 stopped) for delay to centre of first bit after START edge.
#define bit_time 102				//(104-2 for TMR0 stopped) for bits.
#define UART_mask 0b00010000	//to mask in the RA4/T0CKI bit

void interrupt UART(void)
{
static unsigned char UART_mask_byte;

extern unsigned char UART_input_byte;
extern  struct
{
//...other flags
unsigned UART_data_valid	: 1;	//this must be cleared outside somewhere.
unsigned bit_value			: 1;
//...yet more other flags
}flags;

if(T0IF && T0IE)	//we're listening!
	{
	if(T0CS == 1)	//we're waiting for a start transition
		{
		TMR0 -= bit_time_and_a_half;	//the clock's stopped while we do this:
		OPTION = fast_OPTION_config;
		UART_input_byte = 0;	//ready for ORing with this:
		UART_mask_byte = 1;		//LSBit is sent first
		}
	else //we're ticking at bit rate, so it's a bit timeout:
		{
		TMR0 -= bit_time;	//set up for another bit time
		if(RA4 == 1)		//copy the bit value.
			flags.bit_value = 1;
		else
			flags.bit_value = 0;

		if(UART_mask_byte != 0)	//we're in the middle of a byte:
			{
			if(flags.bit_value == 1)	//set the bit in the buffer...
				UART_input_byte |= UART_mask_byte;
//...otherwise don't set the bit, but carry on:
			UART_mask_byte = (UART_mask_byte << 1);	//shift to new bit position
			}	//end of mask_byte not zero

		else	//mask byte is zero, so set up TMR0 for a new START:
			{
			if(flags.bit_value == 1)	//it is the valid stop bit, so:
				flags.UART_data_valid = 1;	//only if valid, but always set up for a new byte:
			OPTION = start_OPTION_config;
			TMR0 = -1;	//wait for next start edge
			}
		}	//end of bit timeout
	T0IF = 0; 	//clear the interrupt
	}	//end of if TMR0IF

}	//end of ISR
//___________________________________________________________

void process_byte(void)	//in UART_input_byte
{

//do what you want with the freshly-arrived byte here.
flags.UART_data_valid = 0;	//show you've dealt with the data.

}

//___________________________________________________________

